/**
 * Copyright (c) 2015-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals;

import java.io.File;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.jar.Attributes;
import java.util.jar.JarInputStream;
import java.util.jar.Manifest;

/**
 * 
 * @author vnoel
 *
 */
public class BootstrapLauncher {

    /**
     * The first argument on the command line must be the jar of petals-launcher-default.
     * 
     * The rest will be passed to its main method.
     * 
     * @param args
     */
    public static void main(final String[] args) throws Exception {

        final String launcherJar = args[0];

        final int newLength = args.length - 1;
        final Object newArgs = new String[newLength];

        System.arraycopy(args, 1, newArgs, 0, newLength);

        final URL launcherURL = new File(launcherJar).toURI().toURL();

        final String mainClass;

        try (final JarInputStream jis = new JarInputStream(launcherURL.openStream())) {
            final Manifest manifest = jis.getManifest();
            final Attributes mainAtts = manifest.getMainAttributes();
            if (mainAtts.containsKey(Attributes.Name.MAIN_CLASS)) {
                mainClass = mainAtts.getValue(Attributes.Name.MAIN_CLASS);
            } else {
                throw new IllegalArgumentException(
                        "Can't find mainClass attribute in MANIFEST from launcher jar: " + launcherJar);
            }
        }

        // we put the jar of the launcher in a new classloader (separate from the system classloader thus)
        // Note: the launcher will load the microkernel jar
        try (final URLClassLoader cl = new URLClassLoader(new URL[] { launcherURL })) {
            // this is needed because some code (such as stopping the petals server) relies on the context
            // classloader
            Thread.currentThread().setContextClassLoader(cl);
            // and run it
            cl.loadClass(mainClass).getDeclaredMethod("main", String[].class).invoke(null, new Object[] { newArgs });
        }
    }
}
